'use strict';

const h3 = require('h3');
const testUtils = require('@vue/test-utils');
const vue = require('vue');
const defu = require('defu');
const _imports = require('#imports');
const NuxtRoot = require('#build/root-component.mjs');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const NuxtRoot__default = /*#__PURE__*/_interopDefaultCompat(NuxtRoot);

function registerEndpoint(url, options) {
  const app = window.__app;
  if (!app)
    return;
  const config = typeof options === "function" ? {
    handler: options,
    method: void 0
  } : options;
  app.use("/_" + url, h3.defineEventHandler(config.handler), {
    match(_, event) {
      return config.method ? event?.method === config.method : true;
    }
  });
  window.__registry.add(url);
}
function mockNuxtImport(_name, _factory) {
  throw new Error(
    "mockNuxtImport() is a macro and it did not get transpiled, this may be an internal bug of nuxt-vitest."
  );
}
function mockComponent(_path, _component) {
  throw new Error(
    "mockComponent() is a macro and it did not get transpiled, this may be an internal bug of nuxt-vitest."
  );
}

const RouterLink = _imports.defineComponent({
  functional: true,
  props: {
    to: [String, Object],
    custom: Boolean,
    replace: Boolean,
    // Not implemented
    activeClass: String,
    exactActiveClass: String,
    ariaCurrentValue: String
  },
  setup: (props, { slots }) => {
    const navigate = () => {
    };
    return () => {
      const route = props.to ? _imports.useRouter().resolve(props.to) : {};
      return props.custom ? slots.default?.({ href: props.to, navigate, route }) : _imports.h(
        "a",
        {
          href: props.to,
          onClick: (e) => {
            e.preventDefault();
            return navigate();
          }
        },
        slots
      );
    };
  }
});

async function mountSuspended(component, options) {
  const {
    props = {},
    attrs = {},
    slots = {},
    route = "/",
    ..._options
  } = options || {};
  const vueApp = globalThis.__unctx__.get("nuxt-app").tryUse().vueApp;
  const { render, setup } = component;
  let setupContext;
  return new Promise((resolve) => {
    const vm = testUtils.mount(
      {
        setup: (props2, ctx) => {
          setupContext = ctx;
          return NuxtRoot__default.setup(props2, {
            ...ctx,
            expose: () => {
            }
          });
        },
        render: (renderContext) => vue.h(
          vue.Suspense,
          { onResolve: () => vue.nextTick().then(() => resolve(vm)) },
          {
            default: () => vue.h({
              async setup() {
                const router = _imports.useRouter();
                await router.replace(route);
                const clonedComponent = {
                  ...component,
                  render: render ? (_ctx, ...args) => render(renderContext, ...args) : void 0,
                  setup: setup ? (props2) => setup(props2, setupContext) : void 0
                };
                return () => vue.h(clonedComponent, { ...props, ...attrs }, slots);
              }
            })
          }
        )
      },
      defu.defu(_options, {
        slots,
        global: {
          config: {
            globalProperties: vueApp.config.globalProperties
          },
          provide: vueApp._context.provides,
          components: { RouterLink }
        }
      })
    );
  });
}

const WRAPPER_EL_ID = "test-wrapper";
async function renderSuspended(component, options) {
  const {
    props = {},
    attrs = {},
    slots = {},
    route = "/",
    ..._options
  } = options || {};
  const { render: renderFromTestingLibrary } = await import('@testing-library/vue');
  const { vueApp } = globalThis.__unctx__.get("nuxt-app").tryUse();
  const { render, setup } = component;
  let setupContext;
  return new Promise((resolve) => {
    const utils = renderFromTestingLibrary(
      {
        // eslint-disable-next-line @typescript-eslint/no-shadow
        setup: (props2, ctx) => {
          setupContext = ctx;
          return NuxtRoot__default.setup(props2, {
            ...ctx,
            expose: () => {
            }
          });
        },
        render: (renderContext) => (
          // See discussions in https://github.com/testing-library/vue-testing-library/issues/230
          // we add this additional root element because otherwise testing-library breaks
          // because there's no root element while Suspense is resolving
          vue.h(
            "div",
            { id: WRAPPER_EL_ID },
            vue.h(
              vue.Suspense,
              { onResolve: () => vue.nextTick().then(() => resolve(utils)) },
              {
                default: () => vue.h({
                  async setup() {
                    const router = _imports.useRouter();
                    await router.replace(route);
                    const clonedComponent = {
                      ...component,
                      render: render ? (_ctx, ...args) => render(renderContext, ...args) : void 0,
                      setup: setup ? (
                        // eslint-disable-next-line @typescript-eslint/no-shadow
                        (props2) => setup(props2, setupContext)
                      ) : void 0
                    };
                    return () => vue.h(clonedComponent, { ...props, ...attrs }, slots);
                  }
                })
              }
            )
          )
        )
      },
      defu.defu(_options, {
        slots,
        global: {
          config: {
            globalProperties: vueApp.config.globalProperties
          },
          provide: vueApp._context.provides,
          components: { RouterLink }
        }
      })
    );
  });
}

exports.mockComponent = mockComponent;
exports.mockNuxtImport = mockNuxtImport;
exports.mountSuspended = mountSuspended;
exports.registerEndpoint = registerEndpoint;
exports.renderSuspended = renderSuspended;
